"""Handler for UUID web application requests"""

import cgi
import re
import wsgiref.handlers

import templates
import uuid_lib

from google.appengine.ext import db
from google.appengine.ext import webapp

UUID_REGEXP = r'\{[0-9a-fA-F]{8}-[0-9a-fA-f]{4}-[0-9a-fA-f]{4}-[0-9a-fA-f]{4}-[0-9a-fA-f]{12}\}$'

class RedirectToMain(webapp.RequestHandler):
  def get(self):
    self.redirect('/uuid/')

class MainPage(webapp.RequestHandler):
  def get(self):
    templates.ReplyWithTemplate(self.response, 'templates/index.html', {})

class QueryResult(webapp.RequestHandler):
  def get(self):
    uuid_query = self.request.get('q').strip().lower()

    db_query = uuid_lib.UuidEntry.all()
    db_query.filter("uuid =", uuid_query)
    results = db_query.fetch(100)
    if len(results) > 0:
      results_text = "";
      uploads = set([uuid.upload_id for uuid in results])
      for upload in sorted(uploads):
        results_text += ("<h2>Found in %s</h2>\n<ul>\n" %
                         cgi.escape(uuid_lib.uploads_map[upload].description))
        upload_uuids = [uuid for uuid in results if uuid.upload_id == upload]
	sort_key = lambda uuid: (uuid.__class__.__name__, uuid.TypeSortKey())
        for uuid in sorted(upload_uuids, key=sort_key):
          results_text += "<li>%s</li>\n" % uuid.HTMLDescription()
        results_text += "</ul>\n"
    else:
      if re.match(UUID_REGEXP, uuid_query):
        results_text = ("<p><i>No results.</i></p>")
      else:
        results_text = ("<p style='color:red'><i>No results. "
	                "This doesn't look like a valid UUID</i></p>")

    templates.ReplyWithTemplate(
        self.response,
	'templates/results.html',
        {'uuid': cgi.escape(uuid_query, True), 'results': results_text})


application = webapp.WSGIApplication([
  ('/uuid', RedirectToMain),
  ('/uuid/', MainPage),
  ('/uuid/query', QueryResult),
], debug=True)


def main():
  wsgiref.handlers.CGIHandler().run(application)


if __name__ == '__main__':
  main()
