// RegCLSIDs.cpp : Defines the entry point for the console application.
//

//#include "stdafx.h"
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <tchar.h>
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>

#define REG_PATH_MAX 4096
TCHAR path[REG_PATH_MAX];

TCHAR norm_clsid[40];

FILE *out;

BOOL is_clsid(TCHAR *name)
{
    if (!(lstrlen(name) == 36 || (lstrlen(name) == 38 && name[0] == '{' && name[37] == '}')))
        return FALSE;
    if (lstrlen(name) == 38)
    {
        lstrcpy(norm_clsid, name);
    }
    else
    {
        lstrcpy(norm_clsid + 1, name);
        norm_clsid[0] = '{';
        lstrcat(norm_clsid, _T("}"));
    }
    name = &norm_clsid[1];
    if (name[8] != '-' || name[13] != '-' || name[18] != '-' || name[23] != '-')
        return FALSE;
    name[8] = name[13] = name[18] = name[23] = '0';
    for (int i = 0; i < 36; i++)
        if (!isxdigit(name[i]))
            return FALSE;
        else
            name[i] = tolower(name[i]);
    name[8] = name[13] = name[18] = name[23] = '-';
    return TRUE;
}

TCHAR *encode(TCHAR *str)
{
    static TCHAR bufs[16][8 * REG_PATH_MAX];
    static int idx = 0;

    idx = (idx+1)%16;
	TCHAR *w = bufs[idx];
#ifdef UNICODE
    WCHAR *s = str;
#else
    WCHAR strW[REG_PATH_MAX];
    assert(MultiByteToWideChar(CP_ACP, 0, str, -1, strW, REG_PATH_MAX));
    WCHAR *s = strW;
#endif
    while (*s)
    {
        if ((*s >= '0' && *s <= '9') || (*s >= 'A' && *s <= 'Z') || (*s >= 'a' && *s <= 'z')
            || *s == '-' || *s == '_' || *s == '\\')
        {
            *(w++) = *s;
        }
        else if (*s != 4)
        {
            /* FIXME: do it properly for ANSI */
            _stprintf(w, _T("&#x%04x;"), *s);
            w += 8;
        } else  /* invalid char */
        {
            _stprintf(w, _T("&#x003f;"));
            w += 8;
        }
        s++;
    }
    *w = 0;

    return bufs[idx];
}

struct LANGANDCODEPAGE {
  WORD wLanguage;
  WORD wCodePage;
};

BOOL get_product(TCHAR *file, TCHAR **ret_product, TCHAR **ret_version)
{
    char buf[65536];
    if (GetFileVersionInfo(file, 0, 65536, buf))
    {
        UINT cbTranslate;
        LANGANDCODEPAGE  *lpTranslate;
        LPVOID lpBuffer;
        if (!VerQueryValue(buf, _T("\\VarFileInfo\\Translation"), (LPVOID*)&lpTranslate, &cbTranslate))
            return FALSE;
        if (cbTranslate < sizeof(LANGANDCODEPAGE))
            return FALSE;

        TCHAR SubBlock[256];
        UINT dwBytes;
        _sntprintf(SubBlock, 256, _T("\\StringFileInfo\\%04X%04X\\ProductName"), lpTranslate->wLanguage, lpTranslate->wCodePage);
        if (!VerQueryValue(buf, SubBlock,  &lpBuffer,  &dwBytes))
            return FALSE;
        *ret_product = (TCHAR *)realloc(*ret_product, sizeof(TCHAR)*(dwBytes+1));
        lstrcpyn(*ret_product, (TCHAR *)lpBuffer, dwBytes);

        _sntprintf(SubBlock, 256, _T("\\StringFileInfo\\%04x%04x\\ProductVersion"), lpTranslate->wLanguage, lpTranslate->wCodePage);
        if (!VerQueryValue(buf, SubBlock,  &lpBuffer,  &dwBytes))
            return FALSE;
        *ret_version = (TCHAR *)realloc(*ret_version, sizeof(TCHAR)*(dwBytes+1));
        lstrcpyn(*ret_version, (TCHAR *)lpBuffer, dwBytes);

        return TRUE;
    }
    return FALSE;
}


void walk(HKEY hkey)
{
    TCHAR name[REG_PATH_MAX];
    DWORD err;
    int pos = lstrlen(path);
    int idx = 0;
    bool clsid = false, iid = false, typelib = false;
    TCHAR *product = NULL, *version = NULL;

    if (lstrcmpi(path, _T("HKLM\\Software\\Classes\\CLSID")) == 0)
        clsid = true;
    if (lstrcmpi(path, _T("HKLM\\Software\\Classes\\Interface")) == 0)
        iid = true;
/*    if (lstrcmpi(path, L"HKLM\\Software\\Classes\\Typelib") == 0)
        typelib = true;*/

    while ((err = RegEnumKey(hkey, idx, name, REG_PATH_MAX)) == 0)
    {
        HKEY hChild;
        DWORD err = RegOpenKey(hkey, name, &hChild);

        if (err != ERROR_ACCESS_DENIED
#ifndef UNICODE
            && (err != ERROR_FILE_NOT_FOUND)
#endif
	   )
        {
            assert(err == ERROR_SUCCESS);

            assert(pos + 1 + lstrlen(name) < REG_PATH_MAX);
            lstrcat(path, _T("\\"));
            lstrcat(path, name);

            walk(hChild);

            path[pos] = 0;

            if (is_clsid(name))
            {
                if (clsid)
                {
                    LONG cb;
					TCHAR clsName[REG_PATH_MAX], server[MAX_PATH];
                    cb = REG_PATH_MAX;
                    err = RegQueryValue(hChild, NULL, clsName, &cb);
                    assert(err == ERROR_SUCCESS);
                    cb = MAX_PATH;
                    err = RegQueryValue(hChild, _T("InprocServer32"), server, &cb);
                    _ftprintf(out, _T("    <reg-clsid name=\"%s\" uuid=\"%s\">\n"), encode(clsName), norm_clsid);
                    if (err != ERROR_FILE_NOT_FOUND)
                    {
                        assert(err == ERROR_SUCCESS);
                        if (get_product(server, &product, &version))
                        {
                            _ftprintf(out, _T("            <inproc-server file=\"%s\" product=\"%s\" product-version=\"%s\"/>\n"), encode(server), encode(product), encode(version));
                        }
                        else
                        {
                            //assert(FALSE);
                            _ftprintf(out, _T("            <inproc-server file=\"%s\"/>\n"), encode(server));
                        }
                    }
                    _ftprintf(out, _T("    </reg-clsid>\n"));
                }
                else if (iid)
                {
                    LONG cb;
                    TCHAR ifName[MAX_PATH];
                    cb = MAX_PATH;
                    err = RegQueryValue(hChild, NULL, ifName, &cb);
                    assert(err == ERROR_SUCCESS);
                    _ftprintf(out, _T("    <reg-iid name=\"%s\" uuid=\"%s\"/>\n"), encode(ifName), norm_clsid);
                }
                else
                    _ftprintf(out, _T("    <reg-subkey key=\"%s\" uuid=\"%s\"/>\n"), encode(path), norm_clsid);
            }

            RegCloseKey(hChild);
        }
        idx++;
    }

    idx = 0;
    DWORD cbNameLen = REG_PATH_MAX;
    while ((err = RegEnumValue(hkey, idx, name, &cbNameLen, NULL, NULL, NULL, NULL)) == 0)
    {
        assert(err == ERROR_SUCCESS);

        if (is_clsid(name))
            _ftprintf(out, _T("    <reg-value key=\"%s\" uuid=\"%s\"/>\n"), encode(path), norm_clsid);
        idx++;
        cbNameLen = REG_PATH_MAX;
    }

    assert(err == ERROR_NO_MORE_ITEMS || err == ERROR_ACCESS_DENIED);
}

int _tmain(int argc, _TCHAR* argv[])
{
    assert(is_clsid(_T("{13775795-83C3-4248-A6EF-A37882FE4D23}")));
    assert(is_clsid(_T("13775795-83C3-4248-A6EF-A37882FE4D23")));

    out = fopen("h:\\out\\reg.xml", "w");
    fprintf(out, "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n");
    fprintf(out, "<reg name=\"Windows XP Registry\">\n");
    lstrcpy(path, _T("HKLM"));
    walk(HKEY_LOCAL_MACHINE);
    lstrcpy(path, _T("HKCU"));
    walk(HKEY_CURRENT_USER);
    fprintf(out, "</reg>");
    fclose(out);
    getchar();
    return 0;
}

