#!/usr/bin/perl

# TODO: doesn't recognize some UUIDs in ExchForm.h and mfapi.h

#$INCLUDE = "/usr/src/wine-git/include";
#$TITLE = "Wine 1.1.0 include directory";
$INCLUDE = "/windows/vista/Program\ Files/Microsoft\ SDKs/Windows/v6.0/Include/";
$TITLE = "Windows SDK 6.0";

opendir(DIR, $INCLUDE) or die $!;

%declspec = ();

sub dump_guid
{
    $file = shift(@_);
    $name = shift(@_);
    $value = shift(@_);
    print(OUT "    <definition file=\"$file\" name=\"$name\" uuid=\"$value\"/>\n");
}

sub dump_declspec_guid
{
    $file = shift(@_);
    $name = shift(@_);
    $value = shift(@_);
    return if (exists $declspec{"$file $value"});

    $hash{"$file $value"} = 1;
    $extra = " name=\"$name\"" unless $name eq "";
    print(OUT "    <declspec file=\"$file\"$extra uuid=\"$value\"/>\n");
}

sub norm_hex
{
    my $num = shift(@_);
    my $digits = shift(@_);

    $num = lc $num;
    if ($num =~ m/^0+$/)
    {
        $num = "0x0";
    }
    
    if (!($num =~ m/^0x0*([0-9a-f]+)$/))
    {
        die "Number $num in invalid format\n";
    }

    $num = $1;
    if (length($num) > $digits)
    {
        die "Number $1 too long (exp. $digits)";
    }
    while (length($num) < $digits)
    {
        $num = '0'.$num;
    }
    return $num;
}

sub check_guid
{
    my $keyword = shift(@_);
    $_ = shift(@_);
    my $extra = shift(@_);

    return if (m/DEFINE_GUIDEX/);
    return if (m/DEFINE_GUIDNAMED/);
    return if (m/DEFINE_GUIDSTRUCT/);
    return if (m/DEFINE_CODECAPI_GUIDNAMED/);

    if (m/\s*$keyword/)
    {
        $NUM = "\\s*([0-9a-fA-FxX]+)[lL]?\\s*";
        $EXTRA = "";
        $EXTRA = "\\s*\\\".*\\\"\\s*," if $extra;
        if (m/\s*$keyword\s*\(\s*([0-9a-zA-Z_]+)\s*,$EXTRA$NUM,$NUM,$NUM,$NUM,$NUM,$NUM,$NUM,$NUM,$NUM,$NUM,$NUM/)
        {
            $guid = "{".norm_hex($2,8)."-".norm_hex($3,4)."-".norm_hex($4,4)."-".norm_hex($5,2).norm_hex($6,2)."-";
            $guid .= norm_hex($7,2).norm_hex($8,2).norm_hex($9,2).norm_hex($10,2).norm_hex($11,2).norm_hex($12,2);
            $guid .= "}";
            dump_guid($file, $1, $guid);
        }
        else
        {
            print "INVGUID: $file: $_\n";
        }
    }

}

sub check_strguid
{
    my $keyword = shift(@_);
    $_ = shift(@_);

    if (m/\s*$keyword/)
    {
        if (m/\s*$keyword\s*\(\"(.+)\"\s*,\s*([0-9a-zA-Z_]+)\s*\)/)
        {
            $guid = lc $1;
            die "INVGUIDSTR! $guid!" if (!($guid =~ m/[0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{12}/));
            dump_guid($file, $2, "{".$guid."}");
        }
        else
        {
            print "INVGUID: $file: $_\n";
        }
    }

}

# For parameter keyword="DECLSPEC_UUID" will detect lines like:
#   DECLSPEC_UUID("{12345678-90ab-cdef-c000-000000000046}")
# or:
#   DECLSPEC_UUID("{12345678-90ab-cdef-c000-000000000046}") ISomething
sub check_str_declspec
{
    my $keyword = shift(@_);
    $_ = shift(@_);

    return if (m/$keyword\(\)/);

    if (m/\s*$keyword/)
    {
        if (m/\s*$keyword\s*\(\"([^\"]+)\"\s*\)[\s\r\n]*(.*)$/s)
        {
            $guid = lc $1;
            $rest = $2;
            die "INVGUIDSTR! $guid!" if (!($guid =~ m/^[0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{12}$/));
            $ident = "";
            $rest =~ s/^enum[\s]+//;
#            print($_."^^$rest\n");
            if ($rest =~ m/([A-Za-z0-9_]+).*/)
            {
#              print("ident: $1\n");
              $ident = $1
            }
            dump_declspec_guid($file, $ident, "{".$guid."}");
        }
        else
        {
            print "INVGUID: $file: $_\n";
        }
    }

}

# Not used - see main.
sub check_midl_guid
{
    my $keyword = "MIDL_DEFINE_GUID";
    $_ = shift(@_);

    if (m/\s*$keyword/)
    {
        $NUM = "\\s*([0-9a-fA-Fx]+)[lL]?\\s*";
        if (m/\s*$keyword\s*\(\s*[0-9a-zA-Z_]+\s*,\s*([0-9a-zA-Z_]+)\s*,$NUM,$NUM,$NUM,$NUM,$NUM,$NUM,$NUM,$NUM,$NUM,$NUM,$NUM/)
        {
            $guid = "{".norm_hex($2,8)."-".norm_hex($3,4)."-".norm_hex($4,4)."-".norm_hex($5,2).norm_hex($6,2)."-";
            $guid .= norm_hex($7,2).norm_hex($8,2).norm_hex($9,2).norm_hex($10,2).norm_hex($11,2).norm_hex($12,2);
            $guid .= "}";
            dump_guid($file, $1, $guid);
        }
        else
        {
            print "INVGUID: $file: $_\n";
        }
    }

}

# For parameters keyword="DEFINE_OLEGUID", tail= "c000-000000000046" will
# detect lines like:
#   DEFINE_OLEGUID(ISomething, 0x12345678, 0x90ab, 0xcdef)
# as UUID {12345678-90ab-cdef-c000-000000000046} for ISomething
sub check_special
{
    my $keyword = shift(@_);
    $_ = shift(@_);
    my $tail = shift(@_);

    if (m/\s*$keyword/)
    {
        $NUM = "\\s*([0-9a-fA-Fx]+)[lL]?\\s*";
        if (m/\s*$keyword\s*\(\s*([0-9a-zA-Z_]+)\s*,$NUM,$NUM,$NUM/)
        {
            $guid = "{".norm_hex($2,8)."-".norm_hex($3,4)."-".norm_hex($4,4)."-".$tail."}";
            dump_guid($file, $1, $guid);
        }
        else
        {
            print "INVGUID: $file: $_\n";
        }
    }

}

# For parameters keyword="DEFINE_DAOGUID", tail="0000-0010-8000-00aa006d2ea4"
# will detect lines like:
#   DEFINE_DOAGUID(ISomething, 0x12345678)
# as UUID {12345678-0000-0010-8000-00aa006d2ea4} for ISomething
sub check_special2
{
    my $keyword = shift(@_);
    $_ = shift(@_);
    my $tail = shift(@_);

    if (m/\s*$keyword/)
    {
        $NUM = "\\s*([0-9a-fA-Fx]+)[lL]?\\s*";
        if (m/\s*$keyword\s*\(\s*([0-9a-zA-Z_]+)\s*,$NUM/)
        {
            $guid = "{".norm_hex($2,8)."-".$tail."}";
            dump_guid($file, $1, $guid);
        }
        else
        {
            print "INVGUID: $file: $_\n";
        }
    }

}

# Parses lines like:
#   DEFINE_NFW_GUID(GUID_SOMETHING, 100)
sub check_nwf
{
    $_ = shift(@_);
    $keyword = "DEFINE_NWF_GUID";

    if (m/\s*$keyword/)
    {
        $NUM = "\\s*([0-9a-fA-Fx]+)[lL]?\\s*";
        if (m/\s*$keyword\s*\(\s*([0-9a-zA-Z_]+)\s*,$NUM/)
        {
            $guid = "{".sprintf("%08x", $2 + 0x6cb9a43e)."-c45f-4039-9fe6-d08cb057184c}";
            dump_guid($file, $1, $guid);
        }
        else
        {
            print "INVGUID: $file: $_\n";
        }
    }

}

open(OUT, ">sdk.xml");
print(OUT "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n");
print(OUT "<sdk name=\"$TITLE\">\n");

while ($file = readdir(DIR))
{
    next if ($file =~ m/\.c$/);
#    print "X${file}X\n";

#    $file = "codecapi.h";
    open(FILE, "<$INCLUDE/$file");
    while (<FILE>)
    {
        s,/\*.*\*/,,g;
        s,//.*$,,g;
        while (m/[,\(]\s*$/)
        {
#            chop $_;
            s/[\r\n]//g;
            $_ .= <FILE>;
            s,/\*.*\*/,,g;
            s,//.*$,,g;
#            print "CONT: $_";
        }

        next if m/cpp_quote/; # we will find it in the *.h file

        # We hope to find the UUID name in the next line for declspecs.
        $_ .= <FILE> if (m/DECLSPEC_UUID/ || m/MIDL_INTERFACE/);

        check_guid("DEFINE_GUID", "$_", 0);
        check_guid("OUR_GUID_ENTRY", "$_", 0);
        check_guid("EXTERN_GUID", "$_", 0);
        check_guid("DEFINE_CODECAPI_GUID", "$_", 1);
        check_special("DEFINE_OLEGUID", "$_", "c000-000000000046");
        check_special("DEFINE_AVIGUID", "$_", "c000-000000000046");
        check_special("DEFINE_MAPIGUID", "$_", "c000-000000000046");
        check_special2("DEFINE_DAOGUID", "$_", "0000-0010-8000-00aa006d2ea4");
        check_strguid("DEFINE_GUIDSTRUCT", "$_");
        check_str_declspec("DECLSPEC_UUID", "$_");
        check_str_declspec("MIDL_INTERFACE", "$_");
        check_nwf("$_");
#        check_special2("DEFINE_MEDIATYPE_GUID", "$_", "0000-0010-8000-00aa00389b71");
#       Present only in *.c files and we should already have the UUIDs from IDLs
#        check_midl_guid("$_");
    }
    close(FILE);
}

print(OUT "</sdk>\n");
